//
//  SchemScanViewController.m
//  SchemScan
//
//  Created by Christophe Guinchard on 06/06/11.
//  Copyright 2011. All rights reserved.
//

#import "SchemScanViewController.h"
#import "QuartzCore/QuartzCore.h"

@implementation SchemScanViewController

@synthesize scrollView;
@synthesize zoomSlider;
@synthesize nextButton;
@synthesize cancelButton;
@synthesize readyLabel;
@synthesize contentView;
@synthesize distanceTable;
@synthesize symbolList;
@synthesize remainingOperation;
@synthesize pinchGestureRecognizer;
@synthesize selectedSymbolID;
@synthesize validSymbolList;
@synthesize displayedSymbolPointList;

@synthesize selectionLayer;
@synthesize matchedLayer;

@synthesize centerSelection;
@synthesize angleSelection;
@synthesize rectSelection;
@synthesize screenOrientationSize;


#pragma mark -
#pragma mark Application lifecycle

- (void)viewDidLoad {
	
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	// Loading background image
	UIImage *bgImage = [UIImage imageWithContentsOfFile: [[NSBundle mainBundle] pathForResource:@"TIFF" ofType:@"tif"] ];
	CGRect frameSize = CGRectMake(0, 0, bgImage.size.width, bgImage.size.height);
	
	// Initialisation du ScrollView
	scrollView.delegate = self;
	[scrollView setScrollEnabled:YES];
	[scrollView setContentSize:bgImage.size];
	scrollView.minimumZoomScale = 0.1;
	scrollView.maximumZoomScale = 2;
	scrollView.zoomScale = 1;
	
	// Init content view
	contentView = [[UIView alloc] initWithFrame:frameSize];
	[scrollView addSubview:contentView];
	
	// Init background layer
	CALayer *backgroundLayer = [CALayer layer];
	backgroundLayer.frame = CGRectMake(0, 0, bgImage.size.width, bgImage.size.height);
	backgroundLayer.contents = (id) bgImage.CGImage;
	backgroundLayer.zPosition = 1;
	[contentView.layer addSublayer:backgroundLayer];
    
    // Init matched layer
    matchedLayer = [CALayer layer];
    matchedLayer.delegate = self;
    matchedLayer.zPosition = 2;
    matchedLayer.masksToBounds = YES;
    matchedLayer.frame = contentView.frame;
    [contentView.layer addSublayer:matchedLayer];
    
    // Init selection layer
    selectionLayer = [CALayer layer];
    selectionLayer.delegate = self;
    selectionLayer.zPosition = 3;
    selectionLayer.masksToBounds = YES;
    [contentView.layer addSublayer:selectionLayer];

    
	// Gestion des gestes
	pinchGestureRecognizer = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(handlePinches:)];
	[scrollView addGestureRecognizer:pinchGestureRecognizer];
    
    // Init taille de la fenêtre selon l'orientation
    if ([[UIDevice currentDevice] orientation] == UIDeviceOrientationPortrait)
    {
        screenOrientationSize = CGSizeMake(1024, 768);
    }
    else
    {
        screenOrientationSize = CGSizeMake(768, 1024);
    }
	
	// Init interface items
	cancelButton.enabled = FALSE;
    nextButton.enabled = FALSE;
    
    // Init arrays
    validSymbolList             = [[NSMutableArray alloc] init];
    displayedSymbolPointList    = [[NSMutableArray alloc] init];
	
	// Récupération des données
	remainingOperation = 2;
	NSString *ssdtPath = [[NSBundle mainBundle] pathForResource:@"fichierdt" ofType:@"ssdt"];
	[self performSelectorInBackground:@selector(parseSSDT:) withObject:ssdtPath];
	NSString *sssbPath = [[NSBundle mainBundle] pathForResource:@"symboleliste" ofType:@"sssb"];
	[self performSelectorInBackground:@selector(parseSSSB:) withObject:sssbPath];
    
	[pool release];
    [super viewDidLoad];
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}

#pragma mark -
#pragma mark Parsing functions

-(int)	parseSSDT:(NSString*) path{
	
	NSLog(@"Starting parsing the SSDT datas with the file %@.",path);
	
	int errorOpenFile = 0;
    int errorDataSize = 0;
	
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	long tailleDonnee = (long)(contentView.frame.size.width*contentView.frame.size.height);
	unsigned short int *tab;
	tab = malloc(tailleDonnee*sizeof(*tab));
	
	// Récupération des données du fichier
	FILE* fichier = NULL;
	fichier = fopen([path cStringUsingEncoding:NSASCIIStringEncoding],"rb");
	if(!fichier) errorOpenFile = 1;
	fread(tab , sizeof(*tab) , tailleDonnee , fichier);
	fclose(fichier);

	NSMutableArray *dataTable = [[NSMutableArray alloc] init];
	for (int i=0;i<tailleDonnee;i++) [dataTable addObject:[NSNumber numberWithUnsignedInt:*(tab+i)]];
    
    // Vérification de la taille des données
    if ([dataTable count] != contentView.frame.size.width*contentView.frame.size.height) errorDataSize = 1;
    
	distanceTable = [[DT alloc] initWithArray:dataTable withSize:contentView.frame.size];
	[dataTable release];
	
	NSLog(@"Finished parsing the SSDT datas. The size of the DT table is %dx%d.",(int)distanceTable.size.width,(int)distanceTable.size.height);

	if (errorOpenFile)
	{
		NSLog(@"Unvalid SSDT file.");
		readyLabel.title = @"Error !";
		[pool release];
		return 1;
	}
    else if (errorDataSize)
    {
        NSLog(@"The size of the data in the SSDT file doesn't match the size of the loaded TIFF file.");
		readyLabel.title = @"Error !";
		[pool release];
		return 2;
    }
	else
	{
		// Mise à jour du compteur d'opération restantes
		remainingOperation--;
		if(!remainingOperation) readyLabel.title = @"Ready";
		[pool release];
	}

	return 0;
	
}

-(int) parseSSSB:(NSString*) path{
	
	NSLog(@"Starting parsing the SSSB datas with the file %@.",path);
	
	int errorOpenFile = 0;
	int errorValue = 0;
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	// Récupération du contenu du fichier avec chaque ligne dans un NSArray 
	NSString *contenuFichier = [[NSString alloc] initWithContentsOfFile:path];
	if (contenuFichier == nil) errorOpenFile = 1;
	symbolList = [[NSMutableArray alloc] init];

	NSArray *lignes = [contenuFichier componentsSeparatedByString:@"\n"];

	/* Pour tous les symboles */
	for (int i = 0 ; i < [lignes count] ; i++)
	{
		NSAutoreleasePool *loopPool = [[NSAutoreleasePool alloc] init];

		NSArray *valeurs = [[lignes objectAtIndex:i] componentsSeparatedByString:@";"];

		/* On vérifie si on a bien un nombre pair de valeurs */
		if ([valeurs count] % 2 == 0)
		{
			NSMutableArray *pointsSymbol = [[NSMutableArray alloc] init];

			/* Pour tous les points */
			for (int j = 0 ; j < [valeurs count] ; j+=2)
			{
				CGPoint pointCourant = CGPointMake([[valeurs objectAtIndex:j] floatValue], [[valeurs objectAtIndex:j+1] floatValue]);
				[pointsSymbol addObject:[NSValue valueWithCGPoint:pointCourant]];
			}

			Symbol *newSymbol = [[Symbol alloc] initWithArray:pointsSymbol];
			[symbolList addObject:(Symbol*) newSymbol];
			[pointsSymbol release];
			[newSymbol release];
		}
		else errorValue = 1;
        
		[loopPool release];
	}

	[contenuFichier release];


	NSLog(@"Finished parsing the SSSB datas. %d symbols where parsed.",[symbolList count]);
	
	if (errorOpenFile)
	{
		NSLog(@"Unvalid SSSB file.");
		readyLabel.title = @"Error !";
		[pool release];
		return 1;
	}
	else if (errorValue)
	{
		NSLog(@"Unvalid number of values in the SSSB file.");
		readyLabel.title = @"Error !";
		[pool release];
		return 2;
	}
	else
	{
		// Mise à jour du compteur d'opération restantes
		remainingOperation--;
		if(!remainingOperation) readyLabel.title = @"Ready";
		[pool release];
	}
	return 0;
}

#pragma mark -
#pragma mark Matching function

-(void) matchSelection {

    if (!remainingOperation)
    {
        remainingOperation++;
        NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
        
        int i, j,pointsNumber;
        Symbol *currentSymbol;
        CGPoint currentPoint;
        unsigned int currentDistance;
        unsigned long int totalDistance, totalSquareDistance;
        CGFloat currentStandardDeviation;
        
        NSMutableArray *sortedDistanceList = [NSMutableArray array];
        int validSymbolNumber = 3;
        
        // Récupération des distances en parcourant cheque point de chaque symbole
        for (i=0 ; i<[symbolList count] ; i++)
        {
            // Affine transformation
            currentSymbol =  [[Symbol alloc] initWithSymbol:[symbolList objectAtIndex:i]];
            [currentSymbol rotateSymbolWithAngle:angleSelection];
            [currentSymbol resizeSymbolWithSize:rectSelection.size];
            
            totalDistance = 0;
            totalSquareDistance = 0;
            pointsNumber = [currentSymbol getNumberPoints];
            
            for (j=0 ; j<pointsNumber ; j++)
            {
                currentPoint = [currentSymbol getPointWithIndex:j WithOrigin:rectSelection.origin];
                currentDistance = [[distanceTable getDistance:currentPoint] unsignedIntValue];
                totalDistance += currentDistance;
                totalSquareDistance += pow(currentDistance, 2);
            }
            
            [currentSymbol release];

            currentStandardDeviation = sqrt( (totalSquareDistance/pointsNumber) - pow(totalDistance/pointsNumber,2) );
            NSArray *currentValue = [NSArray arrayWithObjects:[NSNumber numberWithInt:i], [NSNumber numberWithFloat:currentStandardDeviation], nil];
            if ([sortedDistanceList count] < validSymbolNumber)
                [sortedDistanceList addObject:currentValue];
            else
                [sortedDistanceList replaceObjectAtIndex:validSymbolNumber-1 withObject:currentValue];
            [sortedDistanceList sortUsingFunction:distancesSort context:NULL];
        }
        
        // Ajout des symbols dans la liste des symboles valides
        [validSymbolList removeAllObjects];
        for (i=0 ; i< [sortedDistanceList count] ; i++)
            [validSymbolList addObject:[[sortedDistanceList objectAtIndex:i] objectAtIndex:0]];
        
        [pool release];
        remainingOperation--;
    }
}

#pragma mark -
#pragma mark Interface managment

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
        
    CGFloat currentZoomScale = scrollView.zoomScale;
    
    if((interfaceOrientation == UIDeviceOrientationLandscapeLeft) || (interfaceOrientation == UIDeviceOrientationLandscapeRight)){
        screenOrientationSize = CGSizeMake(1024, 768);
        selectionLayer.frame = CGRectMake(scrollView.contentOffset.x/currentZoomScale, 
                                          scrollView.contentOffset.y/currentZoomScale, 
                                          screenOrientationSize.width/currentZoomScale, 
                                          screenOrientationSize.height/currentZoomScale);
        
            
    } else {
        selectionLayer.frame = CGRectMake(scrollView.contentOffset.x/currentZoomScale, 
                                          scrollView.contentOffset.y/currentZoomScale, 
                                          screenOrientationSize.width/currentZoomScale, 
                                          screenOrientationSize.height/currentZoomScale);
        screenOrientationSize = CGSizeMake(768, 1024);
    }
    
    // Annulation de la selection
    if ([validSymbolList count] != 0)
    {
        selectedSymbolID = 0;
        [validSymbolList removeAllObjects];
        nextButton.enabled = FALSE;
        cancelButton.enabled = FALSE;
        [selectionLayer setNeedsDisplay];
    }
    
    return YES;
}

-(UIView *) viewForZoomingInScrollView:(UIScrollView *)scrollView {
	return contentView;
}

-(IBAction) zoomAction:(id)sender {
	scrollView.zoomScale = ((([scrollView maximumZoomScale]-[scrollView minimumZoomScale])*[zoomSlider value]) + [scrollView minimumZoomScale]);
}

-(IBAction) nextAction:(id)sender {
    if ([validSymbolList count] != 0)
    {
        selectedSymbolID++;
        if (selectedSymbolID > [validSymbolList count]-1) selectedSymbolID = 0;
        [selectionLayer setNeedsDisplay];
    }
}

-(IBAction) cancelAction:(id)sender {
    if ([validSymbolList count] != 0)
    {
        selectedSymbolID = 0;
        [validSymbolList removeAllObjects];
        nextButton.enabled = FALSE;
        cancelButton.enabled = FALSE;
        [selectionLayer setNeedsDisplay];
    }
}

#pragma mark -
#pragma mark Gestures management

-(void) handlePinches:(UIPinchGestureRecognizer*)paramSender{
    
	/* Début Pinch */
	if (paramSender.state == UIGestureRecognizerStateBegan){
        
        // Ajout des points précédents dans displayedSymbolPointList
        if ([validSymbolList count] != 0)
        {
            Symbol *bestSymbol = [[Symbol alloc] initWithSymbol:[symbolList objectAtIndex:[[validSymbolList objectAtIndex:selectedSymbolID] intValue]]];
            [bestSymbol rotateSymbolWithAngle:angleSelection];
            [bestSymbol resizeSymbolWithSize:rectSelection.size];
            int pointsNumber = [bestSymbol getNumberPoints];
            CGPoint currentPoint;
            
            for (int i = 0 ; i < pointsNumber ; i++)
            {
                currentPoint = [bestSymbol getPointWithIndex:i WithOrigin:rectSelection.origin];
                [displayedSymbolPointList addObject:[NSValue valueWithCGPoint:currentPoint]];
            }
            
            [bestSymbol release];
        }
        
        [matchedLayer setNeedsDisplay];
        
        CGFloat currentZoomScale = scrollView.zoomScale;
        selectionLayer.frame = CGRectMake(scrollView.contentOffset.x/currentZoomScale, 
                                          scrollView.contentOffset.y/currentZoomScale, 
                                          screenOrientationSize.width/currentZoomScale, 
                                          screenOrientationSize.height/currentZoomScale);
        selectedSymbolID = 0;
        
	}
	
	/* Continuation Pinch */
	else if (paramSender.state == UIGestureRecognizerStateChanged){

        // Calcul des données du pinch
        CGPoint finger1 = [paramSender locationOfTouch:0 inView:contentView];
        CGPoint finger2 = [paramSender locationOfTouch:1 inView:contentView];
        CGPoint center = CGPointMake((finger1.x+finger2.x)/2, (finger1.y+finger2.y)/2);
        CGFloat radius = sqrt( pow(finger2.x-finger1.x, 2) + pow(finger2.y-finger1.y,2) ) / 2;
        CGFloat angle = atan2((finger1.y-center.y)/radius,(finger1.x-center.x)/radius);
        
        // Setting des données
        centerSelection = center;
        angleSelection  = angle;
        rectSelection   = CGRectMake(center.x-radius, center.y-radius, radius*2, radius*2);
        
        [self performSelectorInBackground:@selector(matchSelection) withObject:NULL];
        
        // Affichage du calque de sélection
        [selectionLayer setNeedsDisplay];
	}
	
	/* Fin Pinch */
	else if (paramSender.state == UIGestureRecognizerStateEnded){
        nextButton.enabled = TRUE;
        cancelButton.enabled = TRUE;
        [selectionLayer setNeedsDisplay];
	}
    
}

#pragma mark -
#pragma mark Conversions

-(CGPoint) convertContentToSelectionLayer:(CGPoint) contentCoordinate {
    return CGPointMake(contentCoordinate.x-selectionLayer.frame.origin.x,contentCoordinate.y-selectionLayer.frame.origin.y);
}

NSInteger distancesSort(id val1, id val2, void *context)
{
    NSArray *array1 = [NSArray arrayWithArray:val1];
    NSArray *array2 = [NSArray arrayWithArray:val2];
    CGFloat distance1 = [[array1 objectAtIndex:1] floatValue];
    CGFloat distance2 = [[array2 objectAtIndex:1] floatValue];
    
    if (distance1 < distance2)
        return NSOrderedAscending;
    else if (distance1 > distance2)
        return NSOrderedDescending;
    else
        return NSOrderedSame;
}

#pragma mark -
#pragma mark Layer management

- (void)drawLayer:(CALayer *)layer inContext:(CGContextRef)context {

    // Dessiner les cercles de sélection
    if (layer.zPosition == 3 && pinchGestureRecognizer.state == UIGestureRecognizerStateChanged)
    {
        CGPoint center = [self convertContentToSelectionLayer:centerSelection];
        CGFloat radius = (rectSelection.size.width/2);
        CGPoint orientationPoint = CGPointMake(center.x + (radius * cos(angleSelection)), center.y + (radius * sin(angleSelection)));
        
        CGColorRef fillingColor = [UIColor colorWithRed:0.5 green:0.0 blue:1.0 alpha:0.4].CGColor;
        CGContextSaveGState(context);

        CGContextSetLineWidth(context, 6.0);
        CGContextSetStrokeColorWithColor(context, fillingColor);
        CGContextAddArc(context, center.x, center.y, radius, 0, (M_PI * 360) / 180, 0);
        CGContextStrokePath(context);
        
        CGContextSetFillColorWithColor(context, fillingColor);
        CGContextAddArc(context, orientationPoint.x, orientationPoint.y, 30, 0, (M_PI * 360) / 180, 0);
        CGContextFillPath(context);
        
        CGContextRestoreGState(context);
    }
    
    // Dessiner le symbole reconnu
    if (layer.zPosition == 3 && [validSymbolList count] != 0)
    {
        CGColorRef fillingColor = [UIColor colorWithRed:0.0 green:1.0 blue:0.0 alpha:0.4].CGColor;
        CGContextSaveGState(context);
        CGContextSetFillColorWithColor(context, fillingColor);
        
        
        Symbol *bestSymbol = [[Symbol alloc] initWithSymbol:[symbolList objectAtIndex:[[validSymbolList objectAtIndex:selectedSymbolID] intValue]]];
        [bestSymbol rotateSymbolWithAngle:angleSelection];
        [bestSymbol resizeSymbolWithSize:rectSelection.size];
        int pointsNumber = [bestSymbol getNumberPoints];
        CGPoint currentPoint;
        
        for (int i = 0 ; i < pointsNumber ; i++)
        {
            currentPoint = [self convertContentToSelectionLayer:[bestSymbol getPointWithIndex:i WithOrigin:rectSelection.origin]];
            CGContextAddArc(context, currentPoint.x, currentPoint.y, 2, 0, (M_PI * 360) / 180, 0);
            CGContextFillPath(context);
        }
        
        CGContextRestoreGState(context);
        [bestSymbol release];
    }
    
    if (layer.zPosition == 2 && [displayedSymbolPointList count] != 0)
    {
        CGPoint currentPoint;
        CGColorRef fillingColor = [UIColor colorWithRed:1.0 green:0.0 blue:0.0 alpha:1].CGColor;
        CGContextSaveGState(context);
        CGContextSetFillColorWithColor(context, fillingColor);
        
        for (int i=0 ; i<[displayedSymbolPointList count] ; i++)
        {
            currentPoint = [[displayedSymbolPointList objectAtIndex:i] CGPointValue];
            CGContextAddArc(context, currentPoint.x, currentPoint.y, 2, 0, (M_PI * 360) / 180, 0);
            CGContextFillPath(context);
        }
        
        CGContextRestoreGState(context);
    }
}

#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)dealloc {
	[scrollView                 release];
	[contentView                release];
	[zoomSlider                 release];
	[nextButton                 release];
    [cancelButton               release];
	[readyLabel                 release];
	[distanceTable              release];
	[symbolList                 release];
    [pinchGestureRecognizer     release];
    [validSymbolList            release];
    [displayedSymbolPointList   release];
    
    [selectionLayer             release];
    [matchedLayer               release];
    [super dealloc];
}
@end
