//
//  Symbol.m
//  SchemScan
//
//  Created by Christophe Guinchard on 06/06/11.
//  Copyright 2011. All rights reserved.
//

#import "Symbol.h"

@implementation Symbol

@synthesize rectSymbol;
@synthesize pointsSymbol;

-(Symbol*) initWithArray:(NSArray *)initTable {
	self = [super init];
	if(self)
	{
		pointsSymbol = [[NSMutableArray alloc] initWithArray:initTable];
		rectSymbol = CGRectMake(0, 0, 0, 0);
		[self updateRect];
	}
	return self;
}

-(Symbol*) initWithSymbol:(Symbol *)initSymbol {
	self = [super init];
	if(self)
	{
		pointsSymbol = [[NSMutableArray alloc] initWithArray:initSymbol.pointsSymbol];
		rectSymbol = initSymbol.rectSymbol;
		[self updateRect];
	}
	return self;
}

-(void) updateRect {
	
	CGFloat minValueX,maxValueX,minValueY,maxValueY;
	CGPoint currentPoint, newPoint;
	int i;
	
	// Première valeure
	currentPoint = [[pointsSymbol objectAtIndex:0] CGPointValue];
	minValueX = currentPoint.x;
	maxValueX = currentPoint.x;
	minValueY = currentPoint.y;
	maxValueY = currentPoint.y;
	
	// Parcours des points
	for (i = 1 ; i < [pointsSymbol count] ; i++)
	{
		currentPoint = [[pointsSymbol objectAtIndex:i] CGPointValue];
		if(currentPoint.x<minValueX) minValueX = currentPoint.x;
		if(currentPoint.x>maxValueX) maxValueX = currentPoint.x;
		if(currentPoint.y<minValueY) minValueY = currentPoint.y;
		if(currentPoint.y>maxValueY) maxValueY = currentPoint.y;
	}
	
	// Recentrer les points
	if (minValueX != 0 || minValueY != 0)
	{
		for (i = 0 ; i < [pointsSymbol count] ; i++)
		{
			currentPoint = [[pointsSymbol objectAtIndex:i] CGPointValue];
			newPoint = CGPointMake(currentPoint.x-minValueX, currentPoint.y-minValueY);
			[pointsSymbol replaceObjectAtIndex:i withObject:[NSValue valueWithCGPoint:newPoint]];
		}
	}
	
	rectSymbol = CGRectMake(0, 0, maxValueX-minValueX, maxValueY-minValueY);
	
}

-(void) resizeSymbolWithSize:(CGSize)newSize {

	if (newSize.width != 0 && newSize.height != 0)
	{
		CGFloat resizeRatioX = newSize.width / rectSymbol.size.width;
		CGFloat resizeRatioY = newSize.height / rectSymbol.size.height;
		CGPoint currentPoint,newPoint;
		
		for (int i = 0 ; i < [pointsSymbol count] ; i++)
		{
			currentPoint = [[pointsSymbol objectAtIndex:i] CGPointValue];
			newPoint = CGPointMake(currentPoint.x * resizeRatioX, currentPoint.y * resizeRatioY);
			[pointsSymbol replaceObjectAtIndex:i withObject:[NSValue valueWithCGPoint:newPoint]];
		}
		
		rectSymbol.size = newSize;
	}
	
}

-(void) rotateSymbolWithAngle:(CGFloat)angle{

	CGPoint centerPoint = CGPointMake(rectSymbol.size.width/2, rectSymbol.size.height/2);
	CGPoint currentPoint,newPoint;
    
    CGAffineTransform translateTransform = CGAffineTransformMakeTranslation(centerPoint.x, centerPoint.y);
    CGAffineTransform rotationTransform = CGAffineTransformMakeRotation(angle);
    CGAffineTransform customRotation = CGAffineTransformConcat(CGAffineTransformConcat( CGAffineTransformInvert(translateTransform), rotationTransform), translateTransform);
	
	for (int i = 0 ; i < [pointsSymbol count] ; i++)
	{
		currentPoint = [[pointsSymbol objectAtIndex:i] CGPointValue];
        newPoint = CGPointApplyAffineTransform(currentPoint, customRotation);
        [pointsSymbol replaceObjectAtIndex:i withObject:[NSValue valueWithCGPoint:newPoint]];
	}
	
	[self updateRect];
}

- (void)dealloc {
	[pointsSymbol	release];
    [super dealloc];
}

-(CGPoint) getPointWithIndex:(NSUInteger)index{
	return [[pointsSymbol objectAtIndex:index] CGPointValue];
}

-(CGPoint) getPointWithIndex:(NSUInteger)index WithOrigin:(CGPoint)originPoint
{
    CGPoint relativePoint = [[pointsSymbol objectAtIndex:index] CGPointValue];
    return CGPointMake(relativePoint.x + originPoint.x, relativePoint.y + originPoint.y);
}

-(NSUInteger) getNumberPoints{
	return [pointsSymbol count];
}

@end
