/*!
 * \file SamplingClasse.cpp
 * \brief Fichier contenant la classe principale  instancier pour effectuer l'chantillonnage
 *        des symboles.
 * \author Christophe Guinchard
 * \version 1.0
 */

#include "SamplingClasse.h"

SamplingClasse::SamplingClasse(void)
{
}

SamplingClasse::~SamplingClasse(void)
{
	symboleListe.clear();
}

int SamplingClasse::chargerSvg(vector<string> &refvecFiles,
                    const char		*rootDirectory,
					bool			bSearchSubdirectories,
                    const char		*refcstrExtension)
{
	string			strFilePath;             // Chemin
	string			strPattern;              // Structure
	string			strExtension;            // Extension
	string			refcstrRootDirectory = rootDirectory;
	HANDLE			hFile;                   // Handle vers le fichier
	WIN32_FIND_DATA	FileInformation;         // Informations du fichier

	int nbErase = 0;
	int lastChar = refcstrRootDirectory.size()-1;
	while (refcstrRootDirectory[lastChar] == '/' ||
		   refcstrRootDirectory[lastChar] == '\\' )
	{
		nbErase ++;
		lastChar--;
	}

	strPattern = refcstrRootDirectory.substr(0,lastChar+1) + "/*.*";

	hFile = ::FindFirstFile(strPattern.c_str(), &FileInformation);
	if(hFile != INVALID_HANDLE_VALUE)
	{
		do
		{
			if(FileInformation.cFileName[0] != '.')
			{
				strFilePath.erase();
				strFilePath = refcstrRootDirectory.substr(0,lastChar+1) + "/" + FileInformation.cFileName;

				if(FileInformation.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					if(bSearchSubdirectories)
					{
						// Recherche dans les sous dossiers
						int iRC = chargerSvg(refvecFiles,rootDirectory,bSearchSubdirectories,refcstrExtension);
						if(iRC) return iRC;
					}
				}
			else
			{
				// Vrification de l'extension
				strExtension = FileInformation.cFileName;
				strExtension = strExtension.substr(strExtension.rfind(".") + 1);

				//Ajouter le nom du fichier dans le vecteur
				if(strExtension == refcstrExtension)
					refvecFiles.push_back(strFilePath);
				}
			}
		}
		while(::FindNextFile(hFile, &FileInformation) == TRUE);

	// Fermer le handle
	::FindClose(hFile);

	DWORD dwError = ::GetLastError();
	if(dwError != ERROR_NO_MORE_FILES)
		return dwError;
	}

	return 0;
}

int SamplingClasse::extractionFormesSVG(char *path, bool bSearchSubdirectories)
{
	int iRC = 0;
	vector<string> fichierListe;

	// Recherche des fichiers TIF dans le dossier
	iRC = chargerSvg(fichierListe, path, bSearchSubdirectories);
	if(iRC)
	{
		cout << "Error " << iRC << endl;
		return 1;
	}

	// On extrait tous les svg
	for(unsigned int i = 0 ; i < fichierListe.size() ; i++)
	{
		//Nouveau symbole
		SamplingSymbole symboleCourant;

		//Chargement du fichier dans un objet TiXmlDocument
		TiXmlDocument doc(fichierListe.at(i).c_str());
		bool loadOkay = doc.LoadFile();
		if (!loadOkay)
			return 1;

		TiXmlElement *elementCourant = doc.RootElement()->FirstChildElement();

		// Recherche de l'element "g" contenant les donnes  extraire
		while (elementCourant->Value() != (string) "g")
		{
			if (!elementCourant->NextSiblingElement()) return 1;
			elementCourant = elementCourant->NextSiblingElement();
		}

		elementCourant = elementCourant->FirstChildElement();

		do
		{
			if (elementCourant->Value() == (string) "line")
			{
				double x1,y1,x2,y2;
				x1 = atof(elementCourant->Attribute("x1")); if (!elementCourant->Attribute("x1")) return 1;
				y1 = atof(elementCourant->Attribute("y1")); if (!elementCourant->Attribute("y1")) return 1;
				x2 = atof(elementCourant->Attribute("x2")); if (!elementCourant->Attribute("x2")) return 1;
				y2 = atof(elementCourant->Attribute("y2")); if (!elementCourant->Attribute("y2")) return 1;

				//Ajouter la ligne au symbole
				symboleCourant.addLigne(x1,y1,x2,y2);
			}
			else if (elementCourant->Value() == (string) "circle")
			{
				double cx, cy, rayon;
				cx = atof(elementCourant->Attribute("cx")); if (!elementCourant->Attribute("cx")) return 1;
				cy = atof(elementCourant->Attribute("cy")); if (!elementCourant->Attribute("cy")) return 1;
				rayon = atof(elementCourant->Attribute("r")); if (!elementCourant->Attribute("r")) return 1;

				//Ajouter le cercle au symbole
				symboleCourant.addCercle(cx,cy,rayon);
			}
			else if (elementCourant->Value() == (string) "path")
			{
				if(symboleCourant.addPath(elementCourant->Attribute("d"))) return 1;
			}

			elementCourant = elementCourant->NextSiblingElement();

		}
		while (elementCourant);

		//Ajout du symbole  la liste
		symboleListe.push_back(symboleCourant);
		
	}

	return 0;
}

void SamplingClasse::samplingPoints (double distance)
{
	for (unsigned i = 0 ; i < symboleListe.size() ; i++)
		symboleListe.at(i).SamplingPoints(distance);
}

int SamplingClasse::sampling(char *path, double distance, bool bSearchSubdirectories)
{
	//Rcupration des SVG  l'emplacement path
	if(extractionFormesSVG(path,bSearchSubdirectories))
		return 1;

	//Calcul des points du sample
	samplingPoints (distance);

	return 0;
}

int SamplingClasse::ecrireFichierListePoints(char *path)
{
	FILE* fichier = NULL;
	fichier = fopen(path,"w");
	if (fichier == NULL) return 1;

		for (unsigned int j = 0 ; j < symboleListe.size() ; j++)
		{
			for (unsigned int i=0 ; i < symboleListe.at(j).getNombrePoint() ; i++)
			{
				fprintf(fichier, "%6.4f;%6.4f", symboleListe.at(j).getPoint(i).first, symboleListe.at(j).getPoint(i).second);
				if (i != symboleListe.at(j).getNombrePoint()-1)
					fprintf(fichier, ";");
				else if (j != symboleListe.size()-1)
					fprintf(fichier, "\n");
			}
	}

	fclose(fichier);

	return 0;
}