#ifndef SAMPLINGCLASSE_H
#define SAMPLINGCLASSE_H

/*!
 * \file SamplingClasse.h
 * \brief Fichier contenant la classe principale  instancier pour effectuer l'chantillonnage
 *        des symboles.
 * \author Christophe Guinchard
 * \version 1.0
 */

#include <string>
#include <vector>
#include <iostream>
#include <sstream>
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include "SamplingSymbole.h"
#include "tinyxml.h"

using namespace std;

/*! \class SamplingClasse
 * \brief Classe principale  instancier pour effectuer l'chantillonnage
 *        des symboles.
 *
 *  Cette classe est instancie en prcisant le lien vers le dossier
 *  contenant l'ensemble des fichiers SVG  chantillonner. Ceux-ci sont
 *  parss pour gnrer un ensemble d'instances de la classe
 *  SamplingSymbole qui aura elle mme la fonction d'chantillonner les
 *  symboles en une liste de points. Les listes de points de l'ensemble des
 *  instance de la classe SamplingSymbole sont ensuite crites dans un
 *  fichier SSSB avec la mthode ecrireFichierListePoints.
 */
class SamplingClasse
{

private:
	vector <SamplingSymbole> symboleListe;

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe SamplingClasse
     */
	SamplingClasse(void);

    /*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe SamplingClasse
     */
	~SamplingClasse(void);

public:

	/*!
     *  \brief Rcupration de la liste des fichiers SVG
     *
     *  Methode qui permet de rcuprer une liste des fichiers SVG se
	 *  trouvant dans un dossier. Il est egalement possible d'explorer les
     *  sous-dossiers.
	 *
     *  \param refvecFiles : Reference  un vecteur o seront crit le chemin
	 *  vers chaque fichier SVG.
     *  \param refcstrRootDirectory : Chemin vers le dossier contenant
	 *  les fichiers SVG.
     *  \param bSearchSubdirectories : Indique s'il faut chercher dans les
	 *  sous-dossiers.
     *  \param refcstrExtension : Extension des fichiers  rechercher.
     *  \return 0 si le dossier  bien pu tre ouvert, sinon 1.
     */
	int		chargerSvg(vector<string> &refvecFiles,
                    const char		*refcstrRootDirectory,
                    bool			bSearchSubdirectories,
                    const char		*refcstrExtension = "svg");

	/*!
     *  \brief Chargement et extraction de fichiers SVG
     *
     *  Methode qui permet d'extraire des fichiers SVG  partir du chemin
	 *  vers le dossier contenant ceux-ci. Avec la mthode chargerSvg
	 *  elle va rcuprer la liste des fichiers SVG. Puis avec la
	 *  bibliothque tinyxml, les informations sur les formes lmentaires
	 *  sont extrait pour instancier une liste de symboles SamplingSymbole.
	 *
     *  \param path : Chemin vers le dossier contenant
	 *  les fichiers SVG.
     *  \param bSearchSubdirectories : Indique s'il faut chercher dans les
	 *  sous-dossiers.
     *  \return 0 si le dossier  bien pu tre ouvert, sinon 1.
     */
	int		extractionFormesSVG(char *path, bool bSearchSubdirectories);

	/*!
     *  \brief Echantillonnage des points
     *
     *  Methode qui permet d'echantillonner les points pour l'ensemble des
	 *  instances contenu dans son vecteur symboleListe.
	 *
     *  \param distance : Distance fixe entre chaque point  chantillonner.
     */
	void	samplingPoints (double distance);

	/*!
     *  \brief Ralisation des opration d'extraction et de sampling
     *
     *  Methode qui va excuter l'ensemble des mthodes ncessaire pour
	 *  raliser l'chantillonnage des points sur un ensemble de fichiers
	 *  SVG. Elle va appeler pour cela les mthodes extractionFormesSVG
	 *  et samplingPoints.
	 *
     *  \param path : Chemin vers le dossier contenant
	 *  \param distance : Distance fixe entre chaque point  chantillonner.
	 *  les fichiers SVG.
     *  \param bSearchSubdirectories : Indique s'il faut chercher dans les
	 *  sous-dossiers.
     *  \return 0 si le dossier  bien pu tre ouvert, sinon 1.
     */
	int		sampling(char *path, double distance = 1, bool bSearchSubdirectories = false);

	/*!
     *  \brief Ecriture du fichier SSSB
     *
     *  Methode qui va crire les coordonnes de chacun des points
	 *  chantillons
	 *
	 *  \param chemin : Chemin vers le fichier SSSB  crire.
     *  \return 0 si le fichier a correctement t crit, sinon 1.
     */
	int		ecrireFichierListePoints(char *path);
};

#endif // SAMPLINGCLASSE_H