#ifndef SAMPLINGFORMEBEZIERCUBIQUE_H
#define SAMPLINGFORMEBEZIERCUBIQUE_H

/*!
 * \file SamplingFormeBezierCubique.h
 * \brief Reprsentation de la forme d'un bzier Cubique tel que reprsent
 *        dans un fichier SVG.
 * \author Christophe Guinchard
 * \version 1.0
 */

/*! \class SamplingFormeBezierCubique
 * \brief Reprsentation de la forme d'un bzier Cubique tel que reprsent
 *        dans un fichier SVG.
 */
class SamplingFormeBezierCubique
{

private:
	double x1, y1;		// Point de dpart
	double cx1, cy1;	// Point de controle 1
	double cx2, cy2;	// Point de controle 2
	double x2, y2;		// Point d'arrive

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe SamplingFormeBezierCubique
	 *
     *  \param sx1 : Coordonne sur l'axe x du premier point du bzier.
     *  \param sy1 : Coordonne sur l'axe y du premier point du bzier.
     *  \param scx1 : Coordonne sur l'axe x du premier point de controle.
     *  \param scy1 : Coordonne sur l'axe y du premier point de controle.
     *  \param scx2 : Coordonne sur l'axe x du second point de controle.
     *  \param scy2 : Coordonne sur l'axe y du second point de controle.
     *  \param sx2 : Coordonne sur l'axe x du dernier point du bzier.
     *  \param sy2 : Coordonne sur l'axe y du dernier point du bzier.
     */
	SamplingFormeBezierCubique(double sx1, double sy1, double scx1, double scy1, double scx2, double scy2, double sx2, double sy2){
		x1 = sx1;
		y1 = sy1;
		cx1 = scx1;
		cy1 = scy1;
		cx2 = scx2;
		cy2 = scy2;
		x2 = sx2;
		y2 = sy2;
	};

    /*!
     *  \brief Constructeur de recopie
     *
     *  Constructeur de recopie de la classe SamplingFormeBezierCubique
	 *
	 *  \param copie : Instance de la classe SamplingFormeBezierCubique 
	 *  recopier.
     */
	SamplingFormeBezierCubique & SamplingFormeBezierCubique::operator = (const SamplingFormeBezierCubique& copie){
		x1 = copie.x1;
		y1 = copie.y1;
		cx1 = copie.cx1;
		cy1 = copie.cy1;
		cx2 = copie.cx2;
		cy2 = copie.cy2;
		x2 = copie.x2;
		y2 = copie.y2;
		return *this;
	}

	/*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe SamplingFormeBezierCubique
     */
	~SamplingFormeBezierCubique(void){};

public:

    /*!
     *  \brief Getter de la valeur x1
     *
     *  Methode qui permet de rcuprer la valeur de x1.
	 *
	 *  \param x1 : Valeur de x1.
     */
	double getX1(void){return x1;};

    /*!
     *  \brief Getter de la valeur y1
     *
     *  Methode qui permet de rcuprer la valeur de y1.
	 *
	 *  \param y1 : Valeur de y1.
     */
	double getY1(void){return y1;};

    /*!
     *  \brief Getter de la valeur cx1
     *
     *  Methode qui permet de rcuprer la valeur de cx1.
	 *
	 *  \param cx1 : Valeur de cx1.
     */
	double getCX1(void){return cx1;};

    /*!
     *  \brief Getter de la valeur cy1
     *
     *  Methode qui permet de rcuprer la valeur de cy1.
	 *
	 *  \param cy1 : Valeur de cy1.
     */
	double getCY1(void){return cy1;};

    /*!
     *  \brief Getter de la valeur cx2
     *
     *  Methode qui permet de rcuprer la valeur de cx2.
	 *
	 *  \param cx2 : Valeur de cx2.
     */
	double getCX2(void){return cx2;};

    /*!
     *  \brief Getter de la valeur cy2
     *
     *  Methode qui permet de rcuprer la valeur de cy2.
	 *
	 *  \param cy2 : Valeur de cy2.
     */
	double getCY2(void){return cy2;};

    /*!
     *  \brief Getter de la valeur x2
     *
     *  Methode qui permet de rcuprer la valeur de x2.
	 *
	 *  \param x2 : Valeur de x2.
     */
	double getX2(void){return x2;};

    /*!
     *  \brief Getter de la valeur y2
     *
     *  Methode qui permet de rcuprer la valeur de y2.
	 *
	 *  \param y2 : Valeur de y2.
     */
	double getY2(void){return y2;};
	
};

#endif // SAMPLINGFORMEBEZIERCUBIQUE_H