#ifndef SAMPLINGFORMEBEZIERQUADRATIQUE_H
#define SAMPLINGFORMEBEZIERQUADRATIQUE_H

/*!
 * \file SamplingFormeBezierQuadratique.h
 * \brief Reprsentation de la forme d'un bzier Quadratique tel que
 *        reprsent dans un fichier SVG.
 * \author Christophe Guinchard
 * \version 1.0
 */

/*! \class SamplingFormeBezierQuadratique
 * \brief Reprsentation de la forme d'un bzier Quadratique tel que
 *        reprsent dans un fichier SVG.
 */
class SamplingFormeBezierQuadratique
{

private:
	double x1, y1;		// Point de dpart
	double cx1, cy1;	// Point de controle 1
	double x2, y2;		// Point d'arrive

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe SamplingFormeBezierQuadratique
	 *
     *  \param sx1 : Coordonne sur l'axe x du premier point du bzier.
     *  \param sy1 : Coordonne sur l'axe y du premier point du bzier.
     *  \param scx1 : Coordonne sur l'axe x du point de controle.
     *  \param scy1 : Coordonne sur l'axe y du point de controle.
     *  \param sx2 : Coordonne sur l'axe x du dernier point du bzier.
     *  \param sy2 : Coordonne sur l'axe y du dernier point du bzier.
     */
	SamplingFormeBezierQuadratique(double sx1, double sy1, double scx1, double scy1, double sx2, double sy2){
		x1 = sx1;
		y1 = sy1;
		cx1 = scx1;
		cy1 = scy1;
		x2 = sx2;
		y2 = sy2;
	};

    /*!
     *  \brief Constructeur de recopie
     *
     *  Constructeur de recopie de la classe SamplingFormeBezierQuadratique
	 *
	 *  \param copie : Instance de la classe SamplingFormeBezierQuadratique 
	 *  recopier.
     */
	SamplingFormeBezierQuadratique & SamplingFormeBezierQuadratique::operator = (const SamplingFormeBezierQuadratique& copie){
		x1 = copie.x1;
		y1 = copie.y1;
		cx1 = copie.cx1;
		cy1 = copie.cy1;
		x2 = copie.x2;
		y2 = copie.y2;
		return *this;
	}

	/*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe SamplingFormeBezierCubique
     */
	~SamplingFormeBezierQuadratique(void){};

public:

    /*!
     *  \brief Getter de la valeur x1
     *
     *  Methode qui permet de rcuprer la valeur de x1.
	 *
	 *  \param x1 : Valeur de x1.
     */
	double getX1(void){return x1;};

    /*!
     *  \brief Getter de la valeur y1
     *
     *  Methode qui permet de rcuprer la valeur de y1.
	 *
	 *  \param y1 : Valeur de y1.
     */
	double getY1(void){return y1;};

    /*!
     *  \brief Getter de la valeur cx1
     *
     *  Methode qui permet de rcuprer la valeur de cx1.
	 *
	 *  \param cx1 : Valeur de cx1.
     */
	double getCX1(void){return cx1;};

    /*!
     *  \brief Getter de la valeur cy1
     *
     *  Methode qui permet de rcuprer la valeur de cy1.
	 *
	 *  \param cy1 : Valeur de cy1.
     */
	double getCY1(void){return cy1;};

    /*!
     *  \brief Getter de la valeur x2
     *
     *  Methode qui permet de rcuprer la valeur de x2.
	 *
	 *  \param x2 : Valeur de x2.
     */
	double getX2(void){return x2;};

    /*!
     *  \brief Getter de la valeur y2
     *
     *  Methode qui permet de rcuprer la valeur de y2.
	 *
	 *  \param y2 : Valeur de y2.
     */
	double getY2(void){return y2;};
	
};

#endif // SAMPLINGFORMEBEZIERQUADRATIQUE_H