#ifndef SAMPLINGFORMECIRCLE_H
#define SAMPLINGFORMECIRCLE_H

/*!
 * \file SamplingFormeCircle.h
 * \brief Reprsentation de la forme d'un cercle tel que reprsent dans
 *        un fichier SVG.
 * \author Christophe Guinchard
 * \version 1.0
 */

/*! \class SamplingFormeCircle
 * \brief Reprsentation de la forme d'un cercle tel que reprsent dans
 *        un fichier SVG.
 */
class SamplingFormeCircle
{

private:
	double cx, cy, rayon;

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe SamplingFormeCircle
	 *
     *  \param scx : Coordonne sur l'axe x du centre du cercle.
     *  \param scy : Coordonne sur l'axe y du centre du cercle.
     *  \param srayon : Rayon du cercle.
     */
	SamplingFormeCircle(double scx, double scy, double srayon){
		cx = scx;
		cy = scy;
		rayon = srayon;
	};

    /*!
     *  \brief Constructeur de recopie
     *
     *  Constructeur de recopie de la classe SamplingFormeCircle
	 *
	 *  \param copie : Instance de la classe SamplingFormeCircle 
	 *  recopier.
     */
	SamplingFormeCircle & SamplingFormeCircle::operator = (const SamplingFormeCircle& copie){
		cx = copie.cx;
		cy = copie.cy;
		rayon = copie.rayon;
		return *this;
	}

	/*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe SamplingFormeCircle
     */
	~SamplingFormeCircle(void){};

public:

    /*!
     *  \brief Getter de la valeur cx
     *
     *  Methode qui permet de rcuprer la valeur de cx.
	 *
	 *  \param cx : Valeur de cx.
     */
	double getCx(void){return cx;};

    /*!
     *  \brief Getter de la valeur cy
     *
     *  Methode qui permet de rcuprer la valeur de cy.
	 *
	 *  \param cy : Valeur de cy.
     */
	double getCy(void){return cy;};

    /*!
     *  \brief Getter de la valeur du rayon
     *
     *  Methode qui permet de rcuprer la valeur du rayon.
	 *
	 *  \param cx : Valeur du rayon.
     */
	double getRayon(void){return rayon;};

};

#endif // SAMPLINGFORMECIRCLE_H