#ifndef SAMPLINGFORMEELLIPSE_H
#define SAMPLINGFORMEELLIPSE_H

/*!
 * \file SamplingFormeEllipse.h
 * \brief Reprsentation de la forme d'une ellipse tel que reprsent dans
 *        un fichier SVG.
 * \author Christophe Guinchard
 * \version 1.0
 */

/*! \class SamplingFormeEllipse
 * \brief Reprsentation de la forme d'une ellipse tel que reprsent dans
 *        un fichier SVG.
 */
class SamplingFormeEllipse
{

private:
	double	x1, y1;			// Point de dpart.
	double	rx, ry;			// Axes majeur et mineur.
	double	xRotation;		// Angle de rotation  partir de l'axe X.
	bool	largeArcFrag;	// Large arc flag. 0->Moins de 180 degrs, 1->Plus de 180 degrs.
	bool	sweepFlag;		// Sweep frag. 0->Angle dcrment, 1->Angle incrment.
	double	x2, y2;			// Point d'arrive.

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe SamplingFormeEllipse
	 *
     *  \param sx1 : Coordonne sur l'axe x du premier point de l'llipse.
     *  \param sy1 : Coordonne sur l'axe y du premier point de l'llipse.
     *  \param srx : Longueur de l'axe x de l'llipse.
     *  \param sry : Longueur de l'axe y de l'llipse.
     *  \param sxRotation : Angle de rotation de l'llipse sur l'axe x.
     *  \param slargeArcFrag : Valeur du flag de grand arc.
     *  \param ssweepFlag : Valeur du Sweep Frag.
     *  \param sx2 : Coordonne sur l'axe x du dernier point de l'llipse.
     *  \param sy2 : Coordonne sur l'axe y du dernier point de l'llipse.
     */
	SamplingFormeEllipse(double sx1, double sy1, double srx, double sry, double sxRotation, bool slargeArcFrag, bool ssweepFlag, double sx2, double sy2){
		x1				= sx1;
		y1				= sy1;
		rx				= srx;
		ry				= sry;
		xRotation		= sxRotation;
		largeArcFrag	= slargeArcFrag;
		sweepFlag		= ssweepFlag;
		x2				= sx2;
		y2				= sy2;
	};

    /*!
     *  \brief Constructeur de recopie
     *
     *  Constructeur de recopie de la classe SamplingFormeEllipse
	 *
	 *  \param copie : Instance de la classe SamplingFormeEllipse 
	 *  recopier.
     */
	SamplingFormeEllipse & SamplingFormeEllipse::operator = (const SamplingFormeEllipse& copie){
		x1				= copie.x1;
		y1				= copie.y1;
		rx				= copie.rx;
		ry				= copie.ry;
		xRotation		= copie.xRotation;
		largeArcFrag	= copie.largeArcFrag;
		sweepFlag		= copie.sweepFlag;
		x2				= copie.x2;
		y2				= copie.y2;
		return *this;
	}

	/*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe SamplingFormeEllipse
     */
	~SamplingFormeEllipse(void){};

public:

    /*!
     *  \brief Getter de la valeur x1
     *
     *  Methode qui permet de rcuprer la valeur de x1.
	 *
	 *  \param x1 : Valeur de x1.
     */
	double	getX1(void){return x1;};

    /*!
     *  \brief Getter de la valeur y1
     *
     *  Methode qui permet de rcuprer la valeur de y1.
	 *
	 *  \param y1 : Valeur de y1.
     */
	double	getY1(void){return y1;};

    /*!
     *  \brief Getter de la valeur rx
     *
     *  Methode qui permet de rcuprer la valeur de rx.
	 *
	 *  \param rx : Valeur de rx.
     */
	double	getRX(void){return rx;};

    /*!
     *  \brief Getter de la valeur ry
     *
     *  Methode qui permet de rcuprer la valeur de ry.
	 *
	 *  \param ry : Valeur de ry.
     */
	double	getRY(void){return ry;};

    /*!
     *  \brief Getter de la valeur xRotation
     *
     *  Methode qui permet de rcuprer la valeur de xRotation.
	 *
	 *  \param xRotation : Valeur de xRotation.
     */
	double	getXRotation(void){return xRotation;};

    /*!
     *  \brief Getter de la valeur largeArcFrag
     *
     *  Methode qui permet de rcuprer la valeur de largeArcFrag.
	 *
	 *  \param largeArcFrag : Valeur de largeArcFrag.
     */
	bool	getLargeArcFrag(void){return largeArcFrag;};

    /*!
     *  \brief Getter de la valeur sweepFlag
     *
     *  Methode qui permet de rcuprer la valeur de sweepFlag.
	 *
	 *  \param sweepFlag : Valeur de sweepFlag.
     */
	bool	getSweepFlag(void){return sweepFlag;};

    /*!
     *  \brief Getter de la valeur x2
     *
     *  Methode qui permet de rcuprer la valeur de x2.
	 *
	 *  \param x2 : Valeur de x2.
     */
	double	getX2(void){return x2;};

    /*!
     *  \brief Getter de la valeur y2
     *
     *  Methode qui permet de rcuprer la valeur de y2.
	 *
	 *  \param y2 : Valeur de y2.
     */
	double	getY2(void){return y2;};
	
};

#endif // SAMPLINGFORMEELLIPSE_H