#ifndef SAMPLINGFORMELINE_H
#define SAMPLINGFORMELINE_H

/*!
 * \file SamplingFormeLine.h
 * \brief Reprsentation de la forme d'une ligne tel que reprsent dans
 *        un fichier SVG.
 * \author Christophe Guinchard
 * \version 1.0
 */

/*! \class SamplingFormeLine
 * \brief Reprsentation de la forme d'une ligne tel que reprsent dans
 *        un fichier SVG.
 */
class SamplingFormeLine
{

private:
	double x1, x2, y1, y2;

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe SamplingFormeLine
	 *
     *  \param sx1 : Coordonne sur l'axe x du premier point de la ligne.
     *  \param sy1 : Coordonne sur l'axe y du premier point de la ligne.
     *  \param sx2 : Coordonne sur l'axe x du dernier point de la ligne.
     *  \param sy2 : Coordonne sur l'axe y du dernier point de la ligne.
     */
	SamplingFormeLine(double sx1, double sy1, double sx2, double sy2){
		x1 = sx1;
		y1 = sy1;
		x2 = sx2;
		y2 = sy2;
	};

    /*!
     *  \brief Constructeur de recopie
     *
     *  Constructeur de recopie de la classe SamplingFormeLine
	 *
	 *  \param copie : Instance de la classe SamplingFormeLine 
	 *  recopier.
     */
	SamplingFormeLine & SamplingFormeLine::operator = (const SamplingFormeLine& copie){
		x1 = copie.x1;
		y1 = copie.y1;
		x2 = copie.x2;
		y2 = copie.y2;
		return *this;
	}

	/*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe SamplingFormeLine
     */
	~SamplingFormeLine(void){};

public:

    /*!
     *  \brief Getter de la valeur x1
     *
     *  Methode qui permet de rcuprer la valeur de x1.
	 *
	 *  \param x1 : Valeur de x1.
     */
	double getX1(void){return x1;};

    /*!
     *  \brief Getter de la valeur x2
     *
     *  Methode qui permet de rcuprer la valeur de x2.
	 *
	 *  \param x2 : Valeur de x2.
     */
	double getX2(void){return x2;};

    /*!
     *  \brief Getter de la valeur y1
     *
     *  Methode qui permet de rcuprer la valeur de y1.
	 *
	 *  \param y1 : Valeur de y1.
     */
	double getY1(void){return y1;};

    /*!
     *  \brief Getter de la valeur y2
     *
     *  Methode qui permet de rcuprer la valeur de y2.
	 *
	 *  \param y2 : Valeur de y2.
     */
	double getY2(void){return y2;};
	
};

#endif // SAMPLINGFORMELINE_H