/*!
 * \file SamplingSymbole.cpp
 * \brief Reprsentation d'un symbole avec les diverses formes le composant
 *        et les mthodes d'chantillonnage.
 * \author Christophe Guinchard
 * \version 1.0
 */

#include "SamplingSymbole.h"

SamplingSymbole::SamplingSymbole(void)
{
}

SamplingSymbole::~SamplingSymbole(void)
{
	ligneListe.clear();
	cercleListe.clear();
	pointListe.clear();
}

void SamplingSymbole::addLigne(double x1, double y1, double x2, double y2)
{
	SamplingFormeLine nouvelleLigne(x1,y1,x2,y2);
	ligneListe.push_back(nouvelleLigne);
}

void SamplingSymbole::addCercle(double cx, double cy, double rayon)
{
	SamplingFormeCircle nouveauCercle(cx,cy,rayon);
	cercleListe.push_back(nouveauCercle);
}

void SamplingSymbole::addBezierCubique(double x1, double y1, double cx1, double cy1, double cx2, double cy2, double x2, double y2)
{
	SamplingFormeBezierCubique nouveauBCubique(x1,y1,cx1,cy1,cx2,cy2,x2,y2);
	bCubiqueListe.push_back(nouveauBCubique);
}

void SamplingSymbole::addBezierQuadratique(double x1, double y1, double cx1, double cy1, double x2, double y2)
{
	SamplingFormeBezierQuadratique nouveauBQuadratique(x1,y1,cx1,cy1,x2,y2);
	bQuadratiqueListe.push_back(nouveauBQuadratique);
}

void SamplingSymbole::addEllipse(double x1, double y1, double rx, double ry, double xRotation, bool largeArcFrag, bool sweepFlag, double x2, double y2)
{
	SamplingFormeEllipse nouvelleEllipse(x1,y1,rx,ry,xRotation,largeArcFrag,sweepFlag,x2,y2);
	ellipseListe.push_back(nouvelleEllipse);
}

void SamplingSymbole::addPoint(double x, double y)
{
	pair <double, double> nouveauPoint(x,y);
	pointListe.push_back(nouveauPoint);
}

int SamplingSymbole::addPath(const char *strpath)
{
	unsigned int	parcours;
	char			charParcours;
	string			stringPath = strpath;
	vector<string>	elementsPath;

	// On parcours l'ensemble de l'instruction path pour sparer chaque lment par des espaces
	for (parcours = 0 ; parcours < stringPath.size() ; parcours++)
	{
		charParcours = stringPath[parcours];
		if ((charParcours >= 'a' && charParcours <= 'z')
			|| (charParcours >= 'A' && charParcours <= 'Z'))
		{
			stringPath.insert(parcours+1," ");
			stringPath.insert(parcours," ");
			parcours++;
		}
		else if (charParcours == '-')
		{
			stringPath.insert(parcours," ");
			parcours++;
		}
		else if (charParcours == ',')
			stringPath.replace(parcours,1,1,' ');
	}

	//Rcupration des lments dans le vecteur
	istringstream iss(stringPath);
	copy(istream_iterator<string>(iss),
		istream_iterator<string>(),
		back_inserter<vector<string> >(elementsPath));

	double pointInitX, pointInitY, pointCourantX, pointCourantY, pointArriveeX, pointArriveeY, valRelativeX, valRelativeY;
	double pointControle1X, pointControle1Y, pointControle2X, pointControle2Y;
	double axe1, axe2, xrot;
	bool largeArcFlag, sweepFlag;
	char elemPrec ='m';

	//Set position initiale
	string test = elementsPath.at(0);
	if (!elementsPath.at(0).compare("m") && !elementsPath.at(0).compare("M"))
		return 1;
	pointInitX = atof(elementsPath.at(1).c_str());
	pointInitY = atof(elementsPath.at(2).c_str());
	pointCourantX = pointInitX;
	pointCourantY = pointInitY;

	//Recherche selon toutes les commandes possible dans path
	for (parcours=3 ; parcours < elementsPath.size() ; parcours++)
	{
		charParcours = elementsPath.at(parcours)[0];
		valRelativeX = 0;
		valRelativeY = 0;

		switch (charParcours)
		{
			case 'm' :	// Move To (relatif)
				valRelativeX = pointCourantX;
				valRelativeY = pointCourantY;
			case 'M' :	// Move To (absolut)
				pointCourantX = valRelativeX + atof(elementsPath.at(parcours+1).c_str());
				pointCourantY = valRelativeY + atof(elementsPath.at(parcours+2).c_str());
				parcours += 2;
				elemPrec = 'm';
				break;

			case 'l' :	// Line To (relatif)
				valRelativeX = pointCourantX;
				valRelativeY = pointCourantY;
			case 'L' :	// Line To (absolut)
				pointArriveeX = valRelativeX + atof(elementsPath.at(parcours+1).c_str());
				pointArriveeY = valRelativeY + atof(elementsPath.at(parcours+2).c_str());
				addLigne(pointCourantX,pointCourantY,pointArriveeX,pointArriveeY);
				pointCourantX = pointArriveeX;
				pointCourantY = pointArriveeY;
				parcours += 2;
				elemPrec = 'l';
				break;

			case 'v' : // Vertical To (relatif)
				valRelativeY = pointCourantY;
			case 'V' : // Vertical To (absolut)
				pointArriveeY = valRelativeY + atof(elementsPath.at(parcours+1).c_str());
				addLigne(pointCourantX,pointCourantY,pointCourantX,pointArriveeY);
				pointCourantY = pointArriveeY;
				parcours += 1;
				elemPrec = 'v';
				break;

			case 'h' : // Horizontal To (relatif)
				valRelativeX = pointCourantX;
			case 'H' : // Horizontal To (absolut)
				pointArriveeX = valRelativeX + atof(elementsPath.at(parcours+1).c_str());
				addLigne(pointCourantX,pointCourantY,pointArriveeX,pointCourantY);
				pointCourantX = pointArriveeX;
				parcours += 1;
				elemPrec = 'h';
				break;

			case 'c' : // Curve To (relatif)
				valRelativeX = pointCourantX;
				valRelativeY = pointCourantY;
			case 'C' : // Curve To (absolut)
				pointControle1X	= valRelativeX + atof(elementsPath.at(parcours+1).c_str());
				pointControle1Y	= valRelativeY + atof(elementsPath.at(parcours+2).c_str());
				pointControle2X	= valRelativeX + atof(elementsPath.at(parcours+3).c_str());
				pointControle2Y	= valRelativeY + atof(elementsPath.at(parcours+4).c_str());
				pointArriveeX	= valRelativeX + atof(elementsPath.at(parcours+5).c_str());
				pointArriveeY	= valRelativeY + atof(elementsPath.at(parcours+6).c_str());
				addBezierCubique(pointCourantX,pointCourantY
					,pointControle1X,pointControle1Y
					,pointControle2X,pointControle2Y
					,pointArriveeX,pointArriveeY);
				pointCourantX = pointArriveeX;
				pointCourantY = pointArriveeY;
				parcours += 6;
				elemPrec = 'c';
				break;

			case 's' : // Smooth Curve To (relatif)
				valRelativeX = pointCourantX;
				valRelativeY = pointCourantY;
			case 'S' : // Smooth Curve To (absolut)
				if (elemPrec == 'c' || elemPrec == 's')
				{
					pointControle1X	= (2*pointCourantX)-atof(elementsPath.at(parcours-4).c_str());
					pointControle1Y	= (2*pointCourantY)-atof(elementsPath.at(parcours-3).c_str());
				}
				else
				{
					pointControle1X = pointCourantX;
					pointControle1Y = pointCourantY;
				}

				pointControle2X	= valRelativeX + atof(elementsPath.at(parcours+1).c_str());
				pointControle2Y	= valRelativeY + atof(elementsPath.at(parcours+2).c_str());
				pointArriveeX	= valRelativeX + atof(elementsPath.at(parcours+3).c_str());
				pointArriveeY	= valRelativeY + atof(elementsPath.at(parcours+4).c_str());
				addBezierCubique(pointCourantX,pointCourantY
					,pointControle1X,pointControle1Y
					,pointControle2X,pointControle2Y
					,pointArriveeX,pointArriveeY);
				pointCourantX = pointArriveeX;
				pointCourantY = pointArriveeY;
				parcours += 4;
				elemPrec = 's';
				break;

			case 'q' : // Quadratic Bezier Curve To (relatif)
				valRelativeX = pointCourantX;
				valRelativeY = pointCourantY;
			case 'Q' : // Quadratic Bezier Curve To (absolut)
				pointControle1X	= valRelativeX + atof(elementsPath.at(parcours+1).c_str());
				pointControle1Y	= valRelativeY + atof(elementsPath.at(parcours+2).c_str());
				pointArriveeX	= valRelativeX + atof(elementsPath.at(parcours+3).c_str());
				pointArriveeY	= valRelativeY + atof(elementsPath.at(parcours+4).c_str());
				addBezierQuadratique(pointCourantX,pointCourantY
					,pointControle1X,pointControle1Y
					,pointArriveeX,pointArriveeY);
				pointCourantX = pointArriveeX;
				pointCourantY = pointArriveeY;
				parcours += 4;
				elemPrec = 'q';
				break;

			case 't' : // Smooth Quadratic Bezier Curve To (relatif)
				valRelativeX = pointCourantX;
				valRelativeY = pointCourantY;
			case 'T' : // Smooth Quadratic BezierCurve To (absolut)
				if (elemPrec == 'q' || elemPrec == 't')
				{
					pointControle1X	= (2*pointCourantX)-atof(elementsPath.at(parcours-4).c_str());
					pointControle1Y	= (2*pointCourantY)-atof(elementsPath.at(parcours-3).c_str());
				}
				else
				{
					pointControle1X = pointCourantX;
					pointControle1Y = pointCourantY;
				}

				pointArriveeX	= valRelativeX + atof(elementsPath.at(parcours+1).c_str());
				pointArriveeY	= valRelativeY + atof(elementsPath.at(parcours+2).c_str());
				addBezierQuadratique(pointCourantX,pointCourantY
					,pointControle1X,pointControle1Y
					,pointArriveeX,pointArriveeY);
				pointCourantX = pointArriveeX;
				pointCourantY = pointArriveeY;
				parcours += 2;
				elemPrec = 't';
				break;

			case 'a' : // Elliptical Arc (relatif)
				valRelativeX = pointCourantX;
				valRelativeY = pointCourantY;
			case 'A' : // Elliptical Arc (absolut)		 (rx ry x-axis-rotation large-arc-flag sweep-flag x y)
				axe1	= abs(atof(elementsPath.at(parcours+1).c_str()));
				axe2	= abs(atof(elementsPath.at(parcours+2).c_str()));
				xrot	= atof(elementsPath.at(parcours+3).c_str());
				if (elementsPath.at(parcours+4)[0] == '0') largeArcFlag=false; else largeArcFlag=true;
				if (elementsPath.at(parcours+5)[0] == '0') sweepFlag=false; else sweepFlag=true;
				pointArriveeX	= valRelativeX + atof(elementsPath.at(parcours+6).c_str());
				pointArriveeY	= valRelativeY + atof(elementsPath.at(parcours+7).c_str());
				addEllipse(pointCourantX,pointCourantY
					,axe1,axe2
					,xrot
					,largeArcFlag,sweepFlag
					,pointArriveeX,pointArriveeY);
				pointCourantX = pointArriveeX;
				pointCourantY = pointArriveeY;
				parcours += 7;
				elemPrec = 'a';
				break;

			case 'Z' :	// Fermer le trac
				addLigne(pointCourantX,pointCourantY,pointInitX,pointInitY);
				pointCourantX = pointInitX;
				pointCourantY = pointInitY;
				elemPrec = 'z';
				break;

			default :
				return 1;
		}
	}


	return 0;
}

void SamplingSymbole::SamplingPoints (double distance)
{
	SamplingPointsLine(distance);
	SamplingPointsCircle(distance);
	SamplingPointBezierCubique(distance);
	SamplingPointBezierQuadratique(distance);
	SamplingPointsEllispse(distance);
}

void SamplingSymbole::SamplingPointsLine (double distance)
{
	double longueurX, longueurY, longueur, distancePoint, longueurPointCourant;
	double x1, y1, x2, y2, nouvX, nouvY;
	unsigned int nbPoint, j, i;

	if (!distance) distance = 1;

	// Parcour de l'ensemble des lignes
	for (j=0 ; j < ligneListe.size() ; j++)
	{
		x1 = ligneListe.at(j).getX1();
		y1 = ligneListe.at(j).getY1();
		x2 = ligneListe.at(j).getX2();
		y2 = ligneListe.at(j).getY2();

		//Ajouter le premier point
		addPoint(x1,y1);

		longueurX = fabs(x1-x2);
		longueurY = fabs(y1-y2);
		longueur = sqrt( pow(longueurX,2) + pow(longueurY,2) );	// Theorme de Pythagore
		nbPoint = (unsigned int) ((longueur / distance) + 0.5);	// Nombre de points
		if (!nbPoint) nbPoint = 1;
		distancePoint = longueur / nbPoint;						// Nouvelle distance inter points

		// Ligne dans le premier cadrant
		if (x1<=x2 && y1<y2)
		{
			for (i=1 ; i < nbPoint ; i++)
			{
				longueurPointCourant = distancePoint * i;

				// Application du thorme de Thals
				nouvX = x1 + (longueurPointCourant / longueur * longueurX);
				nouvY = y1 + (longueurPointCourant / longueur * longueurY);

				// Ajout du nouveau point
				addPoint(nouvX, nouvY);
			}
		}

		// Ligne dans le second cadrant
		else if (x1<x2 && y1>=y2)
		{
			for (i=1 ; i < nbPoint ; i++)
			{
				longueurPointCourant = distancePoint * i;

				// Application du thorme de Thals
				nouvX = x1 + (longueurPointCourant / longueur * longueurX);
				nouvY = y1 - (longueurPointCourant / longueur * longueurY);

				// Ajout du nouveau point
				addPoint(nouvX, nouvY);
			}
		}

		// Ligne dans le troisieme cadrant
		else if (x1>=x2 && y1>y2)
		{
			for (i=1 ; i < nbPoint ; i++)
			{
				longueurPointCourant = distancePoint * i;

				// Application du thorme de Thals
				nouvX = x1 - (longueurPointCourant / longueur * longueurX);
				nouvY = y1 - (longueurPointCourant / longueur * longueurY);

				// Ajout du nouveau point
				addPoint(nouvX, nouvY);
			}
		}

		// Ligne dans le quatrieme cadrant
		else if (x1>x2 && y1<=y2)
		{
			for (i=1 ; i < nbPoint ; i++)
			{
				longueurPointCourant = distancePoint * i;

				// Application du thorme de Thals
				nouvX = x1 - (longueurPointCourant / longueur * longueurX);
				nouvY = y1 + (longueurPointCourant / longueur * longueurY);

				// Ajout du nouveau point
				addPoint(nouvX, nouvY);
			}
		}

		// Ajout du dernier point
		addPoint(x2,y2);
	}
}

void SamplingSymbole::SamplingPointsCircle (double distance)
{
	double perimetre, angleBase, angleCourant;
	double cx, cy, rayon, nouvX, nouvY;
	unsigned int nbPoint, j, i;

	if (!distance) distance = 1;

	// Parcour de l'ensemble des cercles
	for (j=0 ; j < cercleListe.size() ; j++)
	{
		cx = cercleListe.at(j).getCx();
		cy = cercleListe.at(j).getCy();
		rayon = cercleListe.at(j).getRayon();
		perimetre = 2 * PI * rayon;
		nbPoint = (int) (perimetre / distance);
		if (nbPoint<4) nbPoint = 4;
		angleBase = 360 / nbPoint;
		angleCourant = 0;

		for (i=0 ; angleCourant < 360 ; i++)
		{
			angleCourant = angleBase*i;
			nouvX = cx + (rayon * cos(angleCourant * PI / 180));
			nouvY = cy + (rayon * sin(angleCourant * PI / 180));
			addPoint(nouvX,nouvY);
		}
	}
}

void SamplingSymbole::SamplingPointBezierCubique (double distance)
{
	double	positionInitialeX, positionInitialeY, positionFinaleX, positionFinaleY;
	double	xCourant, yCourant;
	double	pointControle1X, pointControle1Y, pointControle2X, pointControle2Y;
	double	distanceParcouru, distanceTotale, distanceEntrePoints;
	unsigned int i, j, nombrePoint;

	for (i = 0 ; i < bCubiqueListe.size() ; i++)
	{
		//Rcupration des donnes
		positionInitialeX	= bCubiqueListe.at(i).getX1();
		positionInitialeY	= bCubiqueListe.at(i).getY1();
		pointControle1X		= bCubiqueListe.at(i).getCX1();
		pointControle1Y		= bCubiqueListe.at(i).getCY1();
		pointControle2X		= bCubiqueListe.at(i).getCX2();
		pointControle2Y		= bCubiqueListe.at(i).getCY2();
 		positionFinaleX		= bCubiqueListe.at(i).getX2();
		positionFinaleY		= bCubiqueListe.at(i).getY2();

		//Dtermination du nombre de points + distance
		distanceTotale = sqrt(pow(positionInitialeX-positionFinaleX,2)
			+ pow(positionInitialeY-positionFinaleY,2));
		nombrePoint = (int) (distanceTotale / distance) + 1;
		distanceEntrePoints = 1./nombrePoint;

		//Dtermination des points  ajouter
		for(j = 0 ; j < nombrePoint ; j++)
		{
			distanceParcouru = distanceEntrePoints * j;
			xCourant = (positionInitialeX*pow(1-distanceParcouru,3))
				+ (3*pointControle1X*distanceParcouru*pow(1-distanceParcouru,2))
				+ (3*pointControle2X*pow(distanceParcouru,2)*pow(1-distanceParcouru,2))
				+ (positionFinaleX*pow(distanceParcouru,3));
			yCourant = (positionInitialeY*pow(1-distanceParcouru,3))
				+ (3*pointControle1Y*distanceParcouru*pow(1-distanceParcouru,2))
				+ (3*pointControle2Y*pow(distanceParcouru,2)*pow(1-distanceParcouru,2))
				+ (positionFinaleY*pow(distanceParcouru,3));
			addPoint(xCourant,yCourant);
		}
		addPoint(positionFinaleX,positionFinaleY);
	}
}

void SamplingSymbole::SamplingPointBezierQuadratique (double distance)
{
	double	positionInitialeX, positionInitialeY, positionFinaleX, positionFinaleY;
	double	xCourant, yCourant;
	double	pointControle1X, pointControle1Y;
	double	distanceParcouru, distanceTotale, distanceEntrePoints;
	unsigned int i, j, nombrePoint;

	for (i = 0 ; i < bQuadratiqueListe.size() ; i++)
	{
		//Rcupration des donnes
		positionInitialeX	= bQuadratiqueListe.at(i).getX1();
		positionInitialeY	= bQuadratiqueListe.at(i).getY1();
		pointControle1X		= bQuadratiqueListe.at(i).getCX1();
		pointControle1Y		= bQuadratiqueListe.at(i).getCY1();
 		positionFinaleX		= bQuadratiqueListe.at(i).getX2();
		positionFinaleY		= bQuadratiqueListe.at(i).getY2();

		//Dtermination du nombre de points + distance
		distanceTotale = sqrt(pow(positionInitialeX-positionFinaleX,2)
			+ pow(positionInitialeY-positionFinaleY,2));
		nombrePoint = (int) (distanceTotale / distance) + 1;
		distanceEntrePoints = 1./nombrePoint;

		//Dtermination des points  ajouter
		for(j = 0 ; j < nombrePoint ; j++)
		{
			distanceParcouru = distanceEntrePoints * j;
			xCourant = (positionInitialeX*pow(1-distanceParcouru,2))
				+ (2*pointControle1X*distanceParcouru*(1-distanceParcouru))
				+ (positionFinaleX*pow(distanceParcouru,2));
			yCourant = (positionInitialeY*pow(1-distanceParcouru,2))
				+ (2*pointControle1Y*distanceParcouru*(1-distanceParcouru))
				+ (positionFinaleY*pow(distanceParcouru,2));
			addPoint(xCourant,yCourant);
		}
		addPoint(positionFinaleX,positionFinaleY);
	}
}

void SamplingSymbole::SamplingPointsEllispse (double distance)
{
	double	positionInitialeX, positionInitialeY, positionFinaleX, positionFinaleY;
	double	axeMajeur, axeMineur;
	double	angleRotationXDegree;
	bool	largeArcFrag, sweepFlag;
	unsigned int i;
	double calculIntermediaire1, calculIntermediaire2, calculIntermediaire3, calculIntermediaire4;

	for (i = 0 ; i < ellipseListe.size() ; i++)
	{

		//Rcupration des donnes
		positionInitialeX		= ellipseListe.at(i).getX1();
		positionInitialeY		= ellipseListe.at(i).getY1();
		axeMajeur				= ellipseListe.at(i).getRX();
		axeMineur				= ellipseListe.at(i).getRY();
		angleRotationXDegree	= ellipseListe.at(i).getXRotation();
		largeArcFrag			= ellipseListe.at(i).getLargeArcFrag();
		sweepFlag				= ellipseListe.at(i).getSweepFlag();
 		positionFinaleX			= ellipseListe.at(i).getX2();
		positionFinaleY			= ellipseListe.at(i).getY2();

		//Conversion de l'angle en gradien
		double angleRotationXGrad = angleRotationXDegree * PI / 180;

		//Calcul de primeX1 et pimeY1
		double primeX1, primeY1;
		calculIntermediaire1 = (positionInitialeX-positionFinaleX)/2;
		calculIntermediaire2 = (positionInitialeY-positionFinaleY)/2;
		primeX1 = (calculIntermediaire1*cos(angleRotationXGrad))
			+ (calculIntermediaire2*sin(angleRotationXGrad));
		primeY1 = (calculIntermediaire1*(-sin(angleRotationXGrad)))
			+ (calculIntermediaire2*cos(angleRotationXGrad));

		//Calcul de primeCX et primeCY
		double primeCX, primeCY, signe;
		if (largeArcFrag == sweepFlag) signe = -1; else signe = 1;
		calculIntermediaire1 = (pow(axeMajeur,2)*pow(axeMineur,2))
			- (pow(axeMajeur,2)*pow(primeY1,2))
			- (pow(axeMineur,2)*pow(primeX1,2));
		calculIntermediaire1 /= (pow(axeMajeur,2)*pow(primeY1,2))
			+ (pow(axeMineur,2)*pow(primeX1,2));
		calculIntermediaire2 = (axeMajeur*primeY1)/axeMineur;
		calculIntermediaire3 = -(axeMineur*primeX1)/axeMajeur;
		primeCX = sqrt(signe * calculIntermediaire1) * calculIntermediaire2;
		primeCY = sqrt(signe * calculIntermediaire1) * calculIntermediaire3;

		//Calcul du centre de l'ellipse cx et cy
		double cx, cy;
		calculIntermediaire1 = (positionInitialeX+positionFinaleX)/2;
		calculIntermediaire2 = (positionInitialeY+positionFinaleY)/2;
		cx = (primeCX*cos(angleRotationXGrad))
			+ (primeCY*-sin(angleRotationXGrad))
			+ calculIntermediaire1;
		cy = (primeCX*sin(angleRotationXGrad))
			+ (primeCY*cos(angleRotationXGrad))
			+ calculIntermediaire2;

		//Calcul de l'angle de dbut de l'ellipse
		double teta;
		calculIntermediaire1 = (primeX1-primeCX)/axeMajeur;
		calculIntermediaire2 = (primeY1-primeCY)/axeMineur;
		if (calculIntermediaire2 < 0) signe = -1; else signe = 1;
		teta = signe * acos(calculIntermediaire1
			/(sqrt(pow(calculIntermediaire1,2) + pow(calculIntermediaire2,2))));
		teta = teta * 180 / PI;

		//Calcul de l'angle total de l'ellipse
		double deltaTeta;
		calculIntermediaire3 = (-primeX1-primeCX)/axeMajeur;
		calculIntermediaire4 = (-primeY1-primeCY)/axeMineur;
		if ((calculIntermediaire1*calculIntermediaire4)-(calculIntermediaire2*calculIntermediaire3) <0)
			signe = -1; else signe = 1;
		deltaTeta = signe * acos(
			((calculIntermediaire1*calculIntermediaire3)
			+ (calculIntermediaire2*calculIntermediaire4))
			/ ((sqrt(pow(calculIntermediaire1,2)
			+ pow(calculIntermediaire2,2)))*(pow(calculIntermediaire3,2) + pow(calculIntermediaire4,2))));
		deltaTeta = deltaTeta * 180 / PI;

		//Calcul du nombre de points
		double distancePoints, angleEntrePoints;
		int nombrePoint;
		distancePoints = sqrt(pow(abs(positionInitialeX-positionFinaleX),2)
			+ pow(abs(positionInitialeY-positionFinaleY),2));
		nombrePoint = (int) (distancePoints/distance);
		angleEntrePoints = deltaTeta / nombrePoint;

		//Dtermination des points intermidiaires
		double xCourant, yCourant, angleCourant;
		for(int j = 0 ; j < nombrePoint ; j++)
		{
			angleCourant = teta + (angleEntrePoints * j);
			calculIntermediaire1 = axeMajeur*cos(angleCourant*PI/180);
			calculIntermediaire2 = axeMineur*sin(angleCourant*PI/180);
			xCourant  = (calculIntermediaire1*cos(angleRotationXGrad))
				+ (calculIntermediaire2*(-sin(angleRotationXGrad))) + cx;
			yCourant  = (calculIntermediaire1*sin(angleRotationXGrad))
				+ (calculIntermediaire2*cos(angleRotationXGrad)) + cy;
			addPoint(xCourant,yCourant);
		}
		addPoint(positionFinaleX,positionFinaleY);


	}

}