#ifndef TDIMAGEBINAIRE_H
#define TDIMAGEBINAIRE_H

/*!
 * \file TDImageBinaire.h
 * \brief Reprsentation de l'image binaire.         
 * \author Christophe Guinchard
 * \version 1.0
 */

/*! \class TDImageBinaire
 * \brief Classe reprsentant l'image binaire.
 *
 *  Cette classe contient pour chaque pixel l'information binaire indiquant
 *  s'il s'agit d'un pixel plein ou vide.
 */
class TDImageBinaire
{

private:
	bool *pixels;
	unsigned int largeur;
	unsigned int hauteur;

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe TDImageBinaire
     */
	TDImageBinaire(void){};

	/*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe TDImageBinaire
     */
	~TDImageBinaire(void){delete pixels;};

public:

	/*!
     *  \brief Initialisation de l'instance
     *
     *  Methode qui permet d'initialiser la hauteur et largeur de l'instance
     *
     *  \param valLargeur : Valeur de la largeur de l'image.
     *  \param valHauteur : Valeur de la hauteur de l'image.
     */
	void initImageBinaire(unsigned int valLargeur, unsigned int valHauteur){
		pixels = new bool [valLargeur*valHauteur];
		largeur = valLargeur;
		hauteur = valHauteur;};

	/*!
     *  \brief Getter de la valeur d'un pixel.
     *
     *  Methode qui permet d'obtenir la valeur d'un pixel.
     *
     *  \param x : Coordonne sur l'axe x de l'image o se trouve le pixel.
     *  \param y : Coordonne sur l'axe y de l'image o se trouve le pixel.
     *  \return la valeur du pixel.
     */
	inline bool getPixel(unsigned int x, unsigned int y){return pixels[(largeur * y) + x];};

	/*!
     *  \brief Setter de la valeur d'un pixel.
     *
     *  Methode qui permet de changer la valeur d'un pixel.
     *
     *  \param x : Coordonne sur l'axe x de l'image o se trouve le pixel.
     *  \param y : Coordonne sur l'axe y de l'image o se trouve le pixel.
     *  \param val : Nouvelle valeur du pixel.
     */
	inline void setPixel(unsigned int x, unsigned int y, bool val){pixels[(largeur * y) + x] = val;};
	
};

#endif // TDIMAGEBINAIRE_H