#ifndef TDIMAGEMAP_H
#define TDIMAGEMAP_H

/*!
 * \file TDImageMap.h
 * \brief Reprsentation de la carte de distance avec les distances
 *        associes  chaque pixel.         
 * \author Christophe Guinchard
 * \version 1.0
 */

/*! \class TDImageMap
 * \brief Classe reprsentant la carte de distance avec les distances
 *        associes  chaque pixel.  
 *
 *  Cette classe contient pour chaque pixel vide sa distance du pixel plein
 *  le plus proche.
 */
class TDImageMap
{

private:
	unsigned short int *pixels;
	unsigned short int largeur;
	unsigned short int hauteur;

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe TDImageMap
     */
	TDImageMap(void){};

	/*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe TDImageBinaire
     */
	~TDImageMap(void){delete pixels;};

public:

	/*!
     *  \brief Initialisation de l'instance
     *
     *  Methode qui permet d'initialiser la hauteur et largeur de l'instance
     *
     *  \param valLargeur : Valeur de la largeur de l'image.
     *  \param valHauteur : Valeur de la hauteur de l'image.
     */
	void initImageMap(unsigned int valLargeur, unsigned int valHauteur){
		pixels = new unsigned short int [valLargeur*valHauteur];
		largeur = valLargeur;
		hauteur = valHauteur;};

	/*!
     *  \brief Getter de la valeur d'un pixel.
     *
     *  Methode qui permet d'obtenir la valeur d'un pixel.
     *
     *  \param x : Coordonne sur l'axe x de l'image o se trouve le pixel.
     *  \param y : Coordonne sur l'axe y de l'image o se trouve le pixel.
     *  \return la valeur du pixel.
     */
	inline unsigned short int getPixel(unsigned int x, unsigned int y){return pixels[(largeur * y) + x];};

	/*!
     *  \brief Getter d'un pointeur vers les valeurs des pixels
     *
     *  Methode qui permet d'obtenir un pointeur vers le premier lment de
	 *  la liste des pixels.
     *
     *  \return le pointeur vers le premier pixel.
     */
	unsigned short int* getPixel(){return pixels;};

	/*!
     *  \brief Getter de la valeur de la hauteur de l'image
     *
     *  Methode qui permet d'obtenir la valeur de la hauteur de l'image.
     *
     *  \return la valeur de la hauteur de l'image.
     */
	unsigned int getLargeur(){return largeur;};

	/*!
     *  \brief Getter de la valeur de la largeur de l'image
     *
     *  Methode qui permet d'obtenir la valeur de la largeur de l'image.
     *
     *  \return la valeur de la largeur de l'image.
     */
	unsigned int getHauteur(){return hauteur;};

	/*!
     *  \brief Setter de la valeur d'un pixel.
     *
     *  Methode qui permet de changer la valeur d'un pixel.
     *
     *  \param x : Coordonne sur l'axe x de l'image o se trouve le pixel.
     *  \param y : Coordonne sur l'axe y de l'image o se trouve le pixel.
     *  \param val : Nouvelle valeur du pixel.
     */
	inline void setPixel(unsigned int x, unsigned int y, unsigned short int val){pixels[(largeur * y) + x] = val;};
	
};

#endif // TDIMAGEMAP_H